@@ -65,8 +65,8 @@ static bool activating(int argc, char **args)
     time(&reg->created);
     if(!reg->setTargets(*addr))
         rtn = false;
-    registry::detach(reg);
     server::activate(reg);
+    registry::detach(reg);
     delete addr;
     return rtn;
 }
@@ -178,6 +178,8 @@ void server::activate(MappedRegistry *rr)
     linked_pointer<modules::sipwitch> cb = getModules();
     logging(rr, "activating");
 
+    events::activate(rr);
+
     while(is(cb)) {
         cb->activating(rr);
         cb.next();
@@ -194,6 +196,7 @@ void server::expire(MappedRegistry *rr)
         cb->expiring(rr);
         cb.next();
     }
+    events::release(rr);
 }
 
 void server::logging(MappedRegistry *rr, const char *reason)
@@ -896,14 +899,14 @@ void server::reload(void)
     }
 
     FILE *fp = fopen(process::get("config"), "r");
-    shell::log(shell::INFO, "loading config from %s", process::get("config"));
     if(fp)
         if(!cfgp->load(fp)) {
-            shell::log(shell::ERR, "invalid config");
+            shell::log(shell::ERR, "invalid config %s", process::get("config"));
             delete cfgp;
             return;
         }
 
+    shell::log(shell::NOTIFY, "loaded config from %s", process::get("config"));
     cp = cfgp->root.getPointer();
     if(cp)
         shell::log(shell::INFO, "activating for state \"%s\"", cp);
@@ -1206,6 +1209,7 @@ invalid:
             }
             process::printlog("state %s %s\n", state, (const char *)logtime);
             shell::log(shell::NOTIFY, "state changed to %s", state);
+            events::state(state);
             reload();
             continue;
         }
