@@ -536,7 +536,7 @@ void stack::destroy(call *cr)
 
     linked_pointer<segment> sp;
 
-    cr->log();
+    cdr *clog = cr->log();
 
     // we assume access lock was already held when we call this...
 
@@ -572,6 +572,8 @@ void stack::destroy(call *cr)
     delete cr;
     locking.share();
     release(map);
+    if(clog)
+        cdr::post(clog);
 }
 
 void stack::release(MappedCall *map)
@@ -693,7 +695,7 @@ void stack::start(service *cfg)
 
     thread *thr;
     unsigned thidx = 0;
-    shell::log(DEBUG1, "stack starting; %d maps and %d threads at priority %d",
+    shell::log(DEBUG1, "starting sip stack; %d maps and %d threads at priority %d",
         mapped_calls, threading, priority);
     eXosip_init();
 
