@@ -37,6 +37,7 @@ static shell::flagopt helpflag('h',"--help",    _TEXT("display this list"));
 static shell::flagopt althelp('?', NULL, NULL);
 static shell::flagopt backflag('b', "--background", _TEXT("run in background"));
 static shell::numericopt concurrency('c', "--concurrency", _TEXT("process concurrency"), "level");
+static shell::flagopt desktop(0, "--desktop", _TEXT("enable desktop access"));
 static shell::flagopt foreflag('f', "--foreground", _TEXT("run in foreground"));
 #ifdef  HAVE_PWD_H
 static shell::stringopt group('g', "--group", _TEXT("use specified group permissions"), "groupid", "nobody");
@@ -117,6 +118,18 @@ static void versioninfo(void)
 
 static bool errlog(shell::loglevel_t level, const char *text)
 {
+    switch(level) {
+    case shell::WARN:
+        events::warning(text);
+        break;
+    case shell::FAIL:
+        events::terminate(text);
+        break;
+    case shell::ERR:
+        events::failure(text);
+    default:
+        break;
+    }
     modules::errlog(level, text);
     history::add(level, text);
     return false;
@@ -133,15 +146,20 @@ namespace SIPWITCH_NAMESPACE {
             stack::enableDumping();
 
         signals::start();
+        events::start();
         server::run();
 
+        events::terminate("server shutdown");
         signals::stop();
         service::shutdown();
         process::release();
     }
 }
 
-extern int main(int argc, char **argv)
+static bool detached = false;
+static SERVICE_MAIN(main, argc, argv);
+
+PROGRAM_MAIN(argc, argv)
 {
 
     bool daemon = true;
@@ -199,6 +217,7 @@ extern int main(int argc, char **argv)
     process::set("config", DEFAULT_CFGPATH "/sipwitch.conf");
     process::set("controls", DEFAULT_VARPATH "/run/sipwitch");
     process::set("control", DEFAULT_VARPATH "/run/sipwitch/control");
+    process::set("events", DEFAULT_VARPATH "/run/sipwitch/events");
     process::set("config", DEFAULT_CFGPATH "/sipwitch.conf");
     process::set("logfiles", DEFAULT_VARPATH "/log");
     process::set("siplogs", DEFAULT_VARPATH "/log/siptrace.log");
@@ -230,6 +249,7 @@ extern int main(int argc, char **argv)
         process::set("config", _STR(str(pwd->pw_dir) + "/.sipwitchrc"));
         process::set("controls", rundir);
         process::set("control", _STR(str(rundir) + "/control"));
+        process::set("events", _STR(str(rundir) + "/events"));
         process::set("logfiles", rundir);
         process::set("siplogs", _STR(str(rundir) + "/siplogs"));
         process::set("logfile", _STR(str(rundir) + "/logfile"));
@@ -374,7 +394,7 @@ extern int main(int argc, char **argv)
             else
                 pwd = getpwnam(*user);
             if(!pwd)
-                shell::errexit(2, "*** apennine: %s: %s\n", *user,
+                shell::errexit(2, "*** sipw: %s: %s\n", *user,
                     _TEXT("unknown or invalid user id"));
         }
     }
@@ -385,7 +405,7 @@ extern int main(int argc, char **argv)
         else
             grp = getgrnam(*group);
         if(!grp)
-            shell::errexit(2, "*** apennine: %s: %s\n", *group,
+            shell::errexit(2, "*** sipw: %s: %s\n", *group,
                 _TEXT("unknown or invalid group id"));
     }
 
@@ -406,11 +426,11 @@ extern int main(int argc, char **argv)
     endgrent();
     endpwent();
 
+    if(is(desktop))
+        umask(002);
 #endif
 
-    signals::setup();
-
-    fsys::createDir(rundir, 0770);
+    fsys::createDir(rundir, 0775);
     fsys::createDir(prefix, 0770);
     if(fsys::changeDir(prefix))
         shell::errexit(3, "*** sipwitch: %s: %s\n",
@@ -422,11 +442,13 @@ extern int main(int argc, char **argv)
 
     // daemonify process....
     if(daemon) {
-        process::args.detach();
+        process::args.detach(&service_main);
         server::logmode = shell::CONSOLE_LOG;
     }
+    else
+        shell::log("sipwitch", level, server::logmode, &errlog);
 
-    shell::log("sipwitch", level, server::logmode, &errlog);
+    signals::setup();
 
     if(!process::attach())
         shell::errexit(1, "*** sipwitch: %s\n",
@@ -443,6 +465,15 @@ extern int main(int argc, char **argv)
 
     up();
 
-    return 0;
+    PROGRAM_EXIT(0);
+}
+
+// stub code for windows service daemon...
+
+static SERVICE_MAIN(main, argc, argv)
+{
+    detached = true;
+    signals::service("sipwitch");
+    main(argc, argv);
 }
 
