@@ -14,16 +14,45 @@
 // along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include "sipwitch/sipwitch.h"
-#ifndef _MSWINDOWS_
+#include "ucommon/secure.h"
+#ifdef _MSWINDOWS_
+#include <io.h>
+#else
 #include <signal.h>
 #include <pwd.h>
 #include <fcntl.h>
 #endif
 #include <config.h>
 
+#if defined(AF_UNIX) && !defined(_MSWINDOWS_)
+#include <sys/un.h>
+#endif
+
 using namespace SIPWITCH_NAMESPACE;
 using namespace UCOMMON_NAMESPACE;
 
+#ifdef  _MSWINDOWS_
+static char *getpass(const char *prompt)
+{
+    static char buf[128];
+    size_t i;
+
+    fputs(prompt, stderr);
+    fflush(stderr);
+    for (i = 0; i < sizeof(buf) - 1; i++) {
+        buf[i] = fgetc(stdin);
+        if (buf[i] == '\r' || buf[i] == '\n')
+            break;
+        fputs("*", stderr);
+        fflush(stderr);
+    }
+    buf[i] = 0;
+    fputs("\n", stderr);
+    fflush(stderr);
+    return buf;
+}
+#endif
+
 static void capture(void)
 {
 #ifndef _MSWINDOWS_
@@ -93,22 +122,190 @@ static void paddress(struct sockaddr_internet *a1, struct sockaddr_internet *a2)
     printf("%s:%u\n", buf, p2);
 }
 
+static void showrealm(void)
+{
+    fsys_t fs;
+    char buffer[256];
+
+    fsys::open(fs, DEFAULT_CFGPATH "/siprealm", fsys::ACCESS_RDONLY);
+    if(!is(fs))
+        fsys::open(fs, DEFAULT_VARPATH "/lib/sipwitch/uuid", fsys::ACCESS_RDONLY);
+
+    if(!is(fs))
+error:
+        shell::errexit(1, "*** sipwitch: realm: no public realm known\n");
+
+    memset(buffer, 0, sizeof(buffer));
+    fsys::read(fs, buffer, sizeof(buffer) - 1);
+    fsys::close(fs);
+
+    char *cp = strchr(buffer, '\n');
+    if(cp)
+        *cp = 0;
+
+    cp = strchr(buffer, ':');
+    if(cp)
+        *cp = 0;
+
+    if(!buffer[0])
+        goto error;
+
+    printf("%s\n", buffer);
+    exit(0);
+}
+
+static void compute(char **argv)
+{
+    char *realm = NULL;
+    const char *user, *secret;
+    const char *mode = "md5";
+    char buffer[128];
+    string_t digestbuf;
+
+    user = argv[1];
+    if(!user)
+        shell::errexit(3, "*** sipwitch: digest: userid missing\n");
+
+    secret = getpass("Enter new SIP secret: ");
+    if(!secret || !*secret) {
+        printf("no password supplied\n");
+        exit(0);
+    }
+
+    realm = argv[2];
+    if(realm) {
+        if(argv[3])
+            mode = argv[3];
+    }
+    else {
+        fsys_t fs;
+        fsys::open(fs, DEFAULT_CFGPATH "/siprealm", fsys::ACCESS_RDONLY);
+        if(!is(fs))
+            fsys::open(fs, DEFAULT_VARPATH "/lib/sipwitch/uuid", fsys::ACCESS_RDONLY);
+
+        if(!is(fs))
+            shell::errexit(4, "*** sipwitch: digest: no public realm known\n");
+
+        memset(buffer, 0, sizeof(buffer));
+        fsys::read(fs, buffer, sizeof(buffer) - 1);
+        fsys::close(fs);
+
+        char *cp = strchr(buffer, '\n');
+        if(cp)
+            *cp = 0;
+
+        cp = strchr(buffer, ':');
+        if(cp)
+            *(cp++) = 0;
+
+        if(cp && *cp)
+            mode = cp;
+        realm = strdup(buffer);
+    }
+
+    digest_t digest = mode;
+    if(digest.puts((string_t)user + ":" + (string_t)realm + ":" + (string_t)secret))
+        digestbuf = *digest;
+    else
+        shell::errexit(6, "** sipwitch: digest: unsupported computation");
+
+    printf("%s\n", *digestbuf);
+    exit(0);
+}
+
+static void realm(char **argv)
+{
+    char *realm = NULL;
+    const char *mode = NULL;
+    fsys_t fs;
+    char buffer[256];
+    char replace[256];
+    char *cp = NULL;
+    FILE *fp;
+
+#ifdef  _MSWINDOWS_
+    const char *control = "\\\\.\\mailslot\\sipwitch_ctrl";
+#else
+    const char *control = DEFAULT_VARPATH "/run/sipwitch/control";
+#endif
+
+    if(!argv[1])
+        showrealm();
+
+    mode = argv[2];
+    if(!mode)
+        mode = "md5";
+
+    fsys::open(fs, DEFAULT_CFGPATH "/siprealm", fsys::ACCESS_RDONLY);
+    memset(buffer, 0, sizeof(buffer));
+    if(is(fs)) {
+        fsys::read(fs, buffer, sizeof(buffer) - 1);
+        fsys::close(fs);
+        cp = strchr(buffer, ':');
+        if(cp)
+            *(cp++) = 0;
+    }
+
+    realm = argv[1];
+    if(!realm)
+        realm = buffer;
+
+    if(!cp || !*cp)
+        cp = (char *)"md5";
+
+    // make sure we have a valid mode, default is md5...
+    if(!mode && cp && *cp)
+        mode = cp;
+
+    if(!mode)
+        mode = "md5";
+
+    // if unchanged, we leave alone...
+    if(eq(buffer, realm) && eq(cp, mode))
+        goto exit;
+
+    // create replacement realm string...
+    if(eq(mode, "md5"))
+        String::set(replace, sizeof(replace), realm);
+    else
+        snprintf(replace, sizeof(replace), "%s:%s", realm, mode);
+
+    ::remove(DEFAULT_CFGPATH "/siprealm");
+    fsys::create(fs, DEFAULT_CFGPATH "/siprealm", fsys::ACCESS_WRONLY, 0644);
+    if(is(fs)) {
+        fsys::write(fs, replace, strlen(replace));
+        fsys::close(fs);
+    }
+    else
+        shell::errexit(3, "*** sipwitch: realm: root permission required\n");
+
+    // if previous digests cached, clear them as they are now invalid...
+    ::remove(DEFAULT_VARPATH "/lib/sipwitch/digests.db");
+
+    // if server is up, also sync server with realm change...
+    fp = fopen(control, "w");
+    if(fp) {
+        fprintf(fp, "realm %s\n", realm);
+        fclose(fp);
+    }
+
+exit:
+    printf("%s\n", realm);
+    exit(0);
+}
+
 static void status(char **argv)
 {
-    if(argv[1]) {
-        fprintf(stderr, "*** sipwitch: calls: no arguments used\n");
-        exit(-1);
-    }
+    if(argv[1])
+        shell::errexit(1, "*** sipwitch: status: no arguments used\n");
 
     mapped_view<MappedCall> calls(CALL_MAP);
     unsigned count = calls.getCount();
     unsigned index = 0;
     const volatile MappedCall *map;
 
-    if(!count) {
-        fprintf(stderr, "*** sipwitch: offline\n");
-        exit(-1);
-    }
+    if(!count)
+        shell::errexit(10, "*** sipwitch: status: offline\n");
 
     while(index < count) {
         map = (const volatile MappedCall *)(calls(index++));
@@ -124,20 +321,17 @@ static void status(char **argv)
 
 static void calls(char **argv)
 {
-    if(argv[1]) {
-        fprintf(stderr, "*** sipwitch: calls: no arguments used\n");
-        exit(-1);
-    }
+    if(argv[1])
+        shell::errexit(1, "*** sipwitch: calls: no arguments used\n");
+
     mapped_view<MappedCall> calls(CALL_MAP);
     unsigned count = calls.getCount();
     unsigned index = 0;
     const volatile MappedCall *map;
     time_t now;
 
-    if(!count) {
-        fprintf(stderr, "*** sipwitch: offline\n");
-        exit(-1);
-    }
+    if(!count)
+        shell::errexit(10, "*** sipwitch: calls: offline\n");
 
     time(&now);
     while(index < count) {
@@ -158,19 +352,17 @@ static void periodic(char **argv)
 {
     char text[80];
 
-    if(argv[1]) {
-        fprintf(stderr, "*** sipwitch: pstats: no arguments used\n");
-        exit(-1);
-    }
+    if(argv[1])
+        shell::errexit(1, "*** sipwitch: pstats: no arguments used\n");
+
     mapped_view<stats> sta(STAT_MAP);
     unsigned count = sta.getCount();
     unsigned index = 0;
     const volatile stats *map;
 
-    if(!count) {
-        fprintf(stderr, "*** sipwitch: offline\n");
-        exit(-1);
-    }
+    if(!count)
+        shell::errexit(10, "*** sipwitch: pstats: offline\n");
+
     while(index < count) {
         map = (const volatile stats *)(sta(index++));
 
@@ -194,15 +386,88 @@ static void periodic(char **argv)
     exit(0);
 }
 
+#if defined(AF_UNIX) && !defined(_MSWINDOWS_)
+static void showevents(char **argv)
+{
+    socket_t ipc = ::socket(AF_UNIX, SOCK_STREAM, 0);
+    struct sockaddr_un addr;
+    struct passwd *pwd = getpwuid(getuid());
+
+    if(argv[1])
+        shell::errexit(1, "*** sipwitch: events: no arguments used\n");
+
+    if(ipc == INVALID_SOCKET)
+        shell::errexit(9, "*** sipwitch: events: cannot create event socket\n");
+
+    memset(&addr, 0, sizeof(addr));
+    addr.sun_family = AF_UNIX;
+    String::set(addr.sun_path, sizeof(addr.sun_path), DEFAULT_VARPATH "/run/sipwitch/events");
+    if(::connect(ipc, (struct sockaddr *)&addr, SUN_LEN(&addr)) < 0) {
+        if(!pwd)
+            shell::errexit(4, "*** sipwitch: events: invalid login\n");
+
+        memset(&addr, 0, sizeof(addr));
+        addr.sun_family = AF_UNIX;
+        snprintf(addr.sun_path, sizeof(addr.sun_path), "/tmp/sipwitch-%s/events", pwd->pw_name);
+        if(::connect(ipc, (struct sockaddr *)&addr, SUN_LEN(&addr)) < 0)
+            shell::errexit(10, "*** sipwitch: events: server offline\n");
+    }
+
+    event_t event;
+    while(::recv(ipc, &event, sizeof(event), 0) == sizeof(event)) {
+        switch(event.type) {
+        case events::FAILURE:
+            printf("failure: %s\n", event.reason);
+            break;
+        case events::WARNING:
+            printf("warning: %s\n", event.reason);
+            break;
+        case events::NOTICE:
+            printf("notice:  %s\n", event.reason);
+            break;
+        case events::WELCOME:
+            printf("server version %s %s\n",
+                event.server.version, event.server.state);
+            break;
+        case events::TERMINATE:
+            printf("exiting: %s\n", event.reason);
+            exit(0);
+        case events::CALL:
+            printf("connecting %s to %s\n",
+                event.call.caller, event.call.dialed);
+            break;
+        case events::DROP:
+            printf("disconnect %s from %s\n",
+                event.call.caller, event.call.dialed);
+            break;
+        case events::RELEASE:
+            if(event.user.extension)
+                printf("releasing %s, extension %d\n",
+                    event.user.id, event.user.extension);
+            else
+                printf("releasing %s\n", event.user.id);
+            break;
+        case events::ACTIVATE:
+            if(event.user.extension)
+                printf("activating %s, extension %d\n",
+                    event.user.id, event.user.extension);
+            else
+                printf("activating %s\n", event.user.id);
+            break;
+        }
+    }
+    shell::errexit(11, "*** sipwitch: events: connection lost\n");
+}
+#endif
+
 static void dumpstats(char **argv)
 {
     char text[80];
     time_t now;
 
-    if(argv[1]) {
-        fprintf(stderr, "*** sipwitch: stats: no arguments used\n");
-        exit(-1);
-    }
+    if(argv[1])
+        shell::errexit(1, "*** sipwitch: stats: no arguments used\n");
+
     mapped_view<stats> sta(STAT_MAP);
     unsigned count = sta.getCount();
     unsigned index = 0;
@@ -210,10 +475,9 @@ static void dumpstats(char **argv)
     unsigned current;
     stats buffer;
 
-    if(!count) {
-        fprintf(stderr, "*** sipwitch: offline\n");
-        exit(-1);
-    }
+    if(!count)
+        shell::errexit(10, "*** sipwitch: stats: offline\n");
+
     time(&now);
     while(index < count) {
         map = const_cast<const stats *>(sta(index++));
@@ -266,15 +530,11 @@ static void registry(char **argv)
     char ext[8], exp[8], use[8];
     const char *type;
 
-    if(argv[1]) {
-        fprintf(stderr, "*** sipwitch: registry: too many arguments\n");
-        exit(-1);
-    }
+    if(argv[1])
+        shell::errexit(1, "*** sipwitch: registry: too many arguments\n");
 
-    if(!count) {
-        fprintf(stderr, "*** sipwitch: offline\n");
-        exit(-1);
-    }
+    if(!count)
+        shell::errexit(10, "*** sipwitch: registry: offline\n");
 
     time(&now);
     while(index < count) {
@@ -348,15 +608,16 @@ static void command(char **argv, unsigned timeout)
 
     fd = ::open(DEFAULT_VARPATH "/run/sipwitch/control", O_WRONLY | O_NONBLOCK);
     if(fd < 0) {
+        if(!pwd)
+            shell::errexit(4, "*** sipwitch: events: invalid login\n");
+
         snprintf(buffer, sizeof(buffer), "/tmp/sipwitch-%s/control", pwd->pw_name);
         fd = ::open(buffer, O_WRONLY | O_NONBLOCK);
     }
 #endif
 
-    if(fd == INVALID_HANDLE_VALUE) {
-        fprintf(stderr, "*** sipwitch: offline\n");
-        exit(2);
-    }
+    if(fd == INVALID_HANDLE_VALUE)
+        shell::errexit(10, "*** sipwitch: command: offline\n");
 
 #ifndef _MSWINDOWS_
     if(timeout)
@@ -371,19 +632,15 @@ static void command(char **argv, unsigned timeout)
     }
 
 #ifdef  _MSWINDOWS_
-    if(!WriteFile(fd, buffer, (DWORD)strlen(buffer) + 1, NULL, NULL)) {
-        fprintf(stderr, "*** sipwitch: control failed\n");
-        exit(4);
-    }
+    if(!WriteFile(fd, buffer, (DWORD)strlen(buffer) + 1, NULL, NULL))
+        shell::errexit(11, "*** sipwitch: control failed\n");
 #else
     len = strlen(buffer);
     buffer[len++] = '\n';
     buffer[len] = 0;
 
-    if(::write(fd, buffer, len) < (int)len) {
-        fprintf(stderr, "*** sipwitch: control failed\n");
-        exit(4);
-    }
+    if(::write(fd, buffer, len) < (int)len)
+        shell::errexit(11, "*** sipwitch: control failed\n");
 
     if(!timeout)
         exit(0);
@@ -398,12 +655,10 @@ static void command(char **argv, unsigned timeout)
         capture();
         exit(0);
     }
-    if(signo == SIGALRM) {
-        fprintf(stderr, "*** sipwitch: timed out\n");
-        exit(1);
-    }
-    fprintf(stderr, "*** sipwitch: request failed\n");
-    exit(3);
+    if(signo == SIGALRM)
+        shell::errexit(12, "*** sipwitch: command: timed out\n");
+
+    shell::errexit(20, "*** sipwitch: command: request failed\n");
 #endif
 }
 
@@ -421,32 +676,37 @@ static void usage(void)
 {
     printf("usage: sipwitch command\n"
         "Commands:\n"
-        "  abort                   Force daemon abort\n"
-        "  activate <ext> <ipaddr> Assign registration\n"
-        "  address <ipaddr>        Set public ip address\n"
-        "  calls                   List active calls on server\n"
-        "  check                   Server deadlock check\n"
-        "  concurrency <level>     Server concurrency level\n"
-        "  down                    Shut down server\n"
-        "  drop <user|callid>      Drop an active call\n"
-        "  dump                    Dump server configuration\n"
-        "  history [bufsize]       Set buffer or dump error log\n"
-        "  ifup <iface>            Notify interface came up\n"
-        "  ifdown <iface>          Notify interface went down\n"
-        "  message <ext> <text>    Send text message to extension\n"
-        "  period <interval>       Collect periodic statistics\n"
-        "  pstats                  Dump periodic statistics\n"
-        "  registry                Dump registry\n"
-        "  release <ext>           Release registration\n"
-        "  reload                  Reload configuration\n"
-        "  restart                 Server restart\n"
-        "  siplog                  Dump sip log when tracing\n"
-        "  snapshot                Server snapshot\n"
-        "  stats                   Dump server statistics\n"
-        "  state <selection>       Change server state\n"
-        "  status                  Dump status string\n"
-        "  trace <on|off|clear>    Set sip message tracing\n"
-        "  verbose <level>         Server verbose logging level\n"
+        "  abort                    Force daemon abort\n"
+        "  activate <ext> <ipaddr>  Assign registration\n"
+        "  address <ipaddr>         Set public ip address\n"
+        "  calls                    List active calls on server\n"
+        "  check                    Server deadlock check\n"
+        "  concurrency <level>      Server concurrency level\n"
+        "  digest id [realm [type]] Compute a digest\n"
+        "  down                     Shut down server\n"
+        "  drop <user|callid>       Drop an active call\n"
+        "  dump                     Dump server configuration\n"
+#if defined(AF_UNIX) && !defined(_MSWINDOWS_)
+        "  events                   Display server events\n"
+#endif
+        "  history [bufsize]        Set buffer or dump error log\n"
+        "  ifup <iface>             Notify interface came up\n"
+        "  ifdown <iface>           Notify interface went down\n"
+        "  message <ext> <text>     Send text message to extension\n"
+        "  period <interval>        Collect periodic statistics\n"
+        "  pstats                   Dump periodic statistics\n"
+        "  realm [text [digest]]    Show or set new server realm\n"
+        "  registry                 Dump registry\n"
+        "  release <ext>            Release registration\n"
+        "  reload                   Reload configuration\n"
+        "  restart                  Server restart\n"
+        "  siplog                   Dump sip log when tracing\n"
+        "  snapshot                 Server snapshot\n"
+        "  stats                    Dump server statistics\n"
+        "  state <selection>        Change server state\n"
+        "  status                   Dump status string\n"
+        "  trace <on|off|clear>     Set sip message tracing\n"
+        "  verbose <level>          Server verbose logging level\n"
     );
 
     printf("Report bugs to sipwitch-devel@gnu.org\n");
@@ -455,120 +715,100 @@ static void usage(void)
 
 static void single(char **argv, int timeout)
 {
-    if(argv[1]) {
-        fprintf(stderr, "*** sipwitch: %s: too many arguments\n", *argv);
-        exit(-1);
-    }
+    if(argv[1])
+        shell::errexit(1, "*** sipwitch: %s: too many arguments\n", *argv);
+
     command(argv, timeout);
 }
 
 static void level(char **argv, int timeout)
 {
-    if(!argv[1]) {
-        fprintf(stderr, "*** sipwitch: %s: level missing\n", *argv);
-        exit(-1);
-    }
-    if(argv[2]) {
-        fprintf(stderr, "*** sipwitch: %s: too many arguments\n", *argv);
-        exit(-1);
-    }
+    if(!argv[1])
+        shell::errexit(1, "*** sipwitch: %s: level missing\n", *argv);
+
+    if(argv[2])
+        shell::errexit(1, "*** sipwitch: %s: too many arguments\n", *argv);
+
     command(argv, timeout);
 }
 
 static void period(char **argv)
 {
-    if(!argv[1]) {
-        fprintf(stderr, "*** sipwitch: period: interval missing\n");
-        exit(-1);
-    }
-    if(argv[2]) {
-        fprintf(stderr, "*** sipwitch: period: too many arguments\n");
-        exit(-1);
-    }
+    if(!argv[1])
+        shell::errexit(1, "*** sipwitch: period: interval missing\n");
+
+    if(argv[2])
+        shell::errexit(1, "*** sipwitch: period: too many arguments\n");
+
     command(argv, 10);
 }
 
 static void address(char **argv)
 {
-    if(!argv[1]) {
-        fprintf(stderr, "*** sipwitch: address: ipaddr missing\n");
-        exit(-1);
-    }
-    if(argv[2]) {
-        fprintf(stderr, "*** sipwitch: address: too many arguments\n");
-        exit(-1);
-    }
+    if(!argv[1])
+        shell::errexit(1, "*** sipwitch: address: ipaddr missing\n");
+
+    if(argv[2])
+        shell::errexit(1, "*** sipwitch: address: too many arguments\n");
+
     command(argv, 10);
 }
 
 static void state(char **argv)
 {
-    if(!argv[1]) {
-        fprintf(stderr, "*** sipwitch: state: value missing\n");
-        exit(-1);
-    }
-    if(argv[2]) {
-        fprintf(stderr, "*** sipwitch: state: too many arguments\n");
-        exit(-1);
-    }
+    if(!argv[1])
+        shell::errexit(1, "*** sipwitch: state: value missing\n");
+
+    if(argv[2])
+        shell::errexit(1, "*** sipwitch: state: too many arguments\n");
+
     command(argv, 10);
 }
 
 static void iface(char **argv)
 {
-    if(!argv[1]) {
-        fprintf(stderr, "*** sipwitch: %s: interface missing\n", *argv);
-        exit(-1);
-    }
-    if(argv[2]) {
-        fprintf(stderr, "*** sipwitch: %s: too many arguments\n", *argv);
-        exit(-1);
-    }
+    if(!argv[1])
+        shell::errexit(1, "*** sipwitch: %s: interface missing\n", *argv);
+
+    if(argv[2])
+        shell::errexit(1, "*** sipwitch: %s: too many arguments\n", *argv);
+
     command(argv, 20);
 }
 
 static void drop(char **argv)
 {
-    if(!argv[1]) {
-        fprintf(stderr, "*** sipwitch: drop: user or callid missing\n");
-        exit(-1);
-    }
-    if(argv[2]) {
-        fprintf(stderr, "*** sipwitch: drop: too many arguments\n");
-        exit(-1);
-    }
+    if(!argv[1])
+        shell::errexit(1, "*** sipwitch: drop: user or callid missing\n");
+
+    if(argv[2])
+        shell::errexit(1, "*** sipwitch: drop: too many arguments\n");
+
     command(argv, 10);
 }
 
 static void release(char **argv)
 {
-    if(!argv[1]) {
-        fprintf(stderr, "*** sipwitch: release: extension missing\n");
-        exit(-1);
-    }
-    if(argv[2]) {
-        fprintf(stderr, "*** sipwitch: release: too many arguments\n");
-        exit(-1);
-    }
+    if(!argv[1])
+        shell::errexit(1, "*** sipwitch: release: extension missing\n");
+
+    if(argv[2])
+        shell::errexit(1, "*** sipwitch: release: too many arguments\n");
+
     command(argv, 10);
 }
 
 static void activate(char **argv)
 {
-    if(!argv[1]) {
-        fprintf(stderr, "*** sipwitch: activate: extension missing\n");
-        exit(-1);
-    }
+    if(!argv[1])
+        shell::errexit(1, "*** sipwitch: activate: extension missing\n");
 
-    if(!argv[2]) {
-        fprintf(stderr, "*** sipwitch: activate: ipaddr missing\n");
-        exit(-1);
-    }
+    if(!argv[2])
+        shell::errexit(1, "*** sipwitch: activate: ipaddr missing\n");
+
+    if(argv[3])
+        shell::errexit(1, "*** sipwitch: activate: too many arguments\n");
 
-    if(argv[3]) {
-        fprintf(stderr, "*** sipwitch: activate: too many arguments\n");
-        exit(-1);
-    }
     command(argv, 10);
 }
 
@@ -576,20 +816,14 @@ static void message(char **argv)
 {
     char buffer[500];
 
-    if(!argv[1]) {
-        fprintf(stderr, "*** sipwitch: message: extension missing\n");
-        exit(-1);
-    }
+    if(!argv[1])
+        shell::errexit(1, "*** sipwitch: message: extension missing\n");
 
-    if(!argv[2]) {
-        fprintf(stderr, "*** sipwitch: message: \"text\" missing\n");
-        exit(-1);
-    }
+    if(!argv[2])
+        shell::errexit(1, "*** sipwitch: message: \"text\" missing\n");
 
-    if(argv[3]) {
-        fprintf(stderr, "*** sipwitch: message: too many arguments\n");
-        exit(-1);
-    }
+    if(argv[3])
+        shell::errexit(1, "*** sipwitch: message: too many arguments\n");
 
     if(argv[2][0] != '{') {
         snprintf(buffer, sizeof(buffer), "{%s}", argv[2]);
@@ -598,7 +832,7 @@ static void message(char **argv)
     command(argv, 10);
 }
 
-extern "C" int main(int argc, char **argv)
+PROGRAM_MAIN(argc, argv)
 {
     if(argc < 2)
         usage();
@@ -628,6 +862,8 @@ extern "C" int main(int argc, char **argv)
         dumpstats(argv);
     else if(eq(*argv, "calls"))
         calls(argv);
+    else if(eq(*argv, "digest"))
+        compute(argv);
     else if(eq(*argv, "pstats"))
         periodic(argv);
     else if(eq(*argv, "address"))
@@ -644,12 +880,18 @@ extern "C" int main(int argc, char **argv)
         status(argv);
     else if(eq(*argv, "ifdown") || eq(*argv, "ifup"))
         iface(argv);
+    else if(eq(*argv, "realm"))
+            realm(argv);
     else if(eq(*argv, "drop"))
         drop(argv);
+#if defined(AF_UNIX) && !defined(_MSWINDOWS_)
+    if(eq(*argv, "events"))
+        showevents(argv);
+#endif
     if(!argv[1])
-        fprintf(stderr, "use: sipwitch command [arguments...]\n");
+        shell::errexit(1, "use: sipwitch command [arguments...]\n");
     else
-        fprintf(stderr, "*** sipwitch: %s: unknown command or option\n", argv[0]);
-    exit(1);
+        shell::errexit(1, "*** sipwitch: %s: unknown command or option\n", argv[0]);
+    PROGRAM_EXIT(1);
 }
 
